@@ -1,7 +1,7 @@
 /*
  * @file itemview.c  viewing feed content in different presentation modes
  *
- * Copyright (C) 2006-2022 Lars Windolf <lars.windolf@gmx.de>
+ * Copyright (C) 2006-2024 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,16 +21,16 @@
 #include <string.h>
 
 #include "conf.h"
+#include "common.h"
 #include "debug.h"
-#include "folder.h"
+#include "feedlist.h"
 #include "item_history.h"
 #include "itemlist.h"
 #include "itemview.h"
 #include "node.h"
-#include "vfolder.h"
+#include "node_provider.h"
 #include "ui/ui_common.h"
 #include "ui/browser_tabs.h"
-#include "ui/enclosure_list_view.h"
 #include "ui/liferea_shell.h"
 #include "ui/item_list_view.h"
 #include "ui/liferea_browser.h"
@@ -47,7 +47,7 @@ struct _ItemView {
 	GObject	parent_instance;
 
 	guint		mode;			/*<< current item view mode */
-	nodePtr		node;			/*<< the node whose items are displayed */
+	Node		*node;			/*<< the node whose items are displayed */
 	gboolean	browsing;		/*<< TRUE if itemview is used as internal browser right now */
 	gboolean	needsHTMLViewUpdate;	/*<< flag to be set when HTML rendering is to be
 						     updated, used to delay HTML updates */
@@ -58,7 +58,6 @@ struct _ItemView {
 
 	ItemListView	*itemListView;		/*<< widget instance used to present items in list mode */
 
-	EnclosureListView	*enclosureView;	/*<< Enclosure list widget */
 	LifereaBrowser		*htmlview;	/*<< HTML rendering widget instance used to render single items and summaries mode */
 
 	gfloat			zoom;		/*<< HTML rendering widget zoom level */
@@ -86,8 +85,6 @@ itemview_finalize (GObject *object)
 		g_object_unref (itemview->htmlview);
 	}
 
-	if (itemview->enclosureView)
-		g_object_unref (itemview->enclosureView);
 	if (itemview->itemListView)
 		g_object_unref (itemview->itemListView);
 }
@@ -144,7 +141,6 @@ itemview_clear (void)
 {
 	if (itemview->itemListView)
 		item_list_view_clear (itemview->itemListView);
-	enclosure_list_view_hide (itemview->enclosureView);
 	itemview->needsHTMLViewUpdate = TRUE;
 	itemview->browsing = FALSE;
 }
@@ -161,7 +157,7 @@ itemview_set_mode (itemViewMode mode)
 }
 
 void
-itemview_set_displayed_node (nodePtr node)
+itemview_set_displayed_node (Node *node)
 {
 	if (node == itemview->node)
 		return;
@@ -171,6 +167,14 @@ itemview_set_displayed_node (nodePtr node)
 	itemview_clear ();
 }
 
+guint itemview_get_effective_layout (void)
+{
+	if (itemview->autoLayout)
+		return itemview->currentLayoutMode;
+
+	return itemview->mode;
+}
+
 void
 itemview_add_item (itemPtr item)
 {
@@ -197,29 +201,10 @@ itemview_select_item (itemPtr item)
 	if (itemview->itemListView)
 		item_list_view_select (itemview->itemListView, item);
 
-	if (item)
-		enclosure_list_view_load (itemview->enclosureView, item);
-	else
-		enclosure_list_view_hide (itemview->enclosureView);
-
 	if (item)
 		item_history_add (item->id);
 }
 
-void
-itemview_select_enclosure (guint position)
-{
-	if (itemview->enclosureView)
-		enclosure_list_view_select (itemview->enclosureView, position);
-}
-
-void
-itemview_open_next_enclosure (ItemView *view)
-{
-	if (view->enclosureView)
-		enclosure_list_view_open_next (view->enclosureView);
-}
-
 void
 itemview_update_item (itemPtr item)
 {
@@ -262,7 +247,7 @@ itemview_update_all_items (void)
 }
 
 void
-itemview_update_node_info (nodePtr node)
+itemview_update_node_info (Node *node)
 {
 	/* Bail if we do internal browsing, and no item is shown */
 	if (itemview->browsing)
@@ -284,17 +269,51 @@ itemview_update_node_info (nodePtr node)
 void
 itemview_update (void)
 {
+	g_autoptr(LifereaItem) 	item = NULL;
+	Node			*node = NULL;
+	g_autofree gchar	*baseURL = NULL;
+	g_autofree gchar 	*json = NULL;
+	const gchar		*direction = NULL;
+	const gchar		*name = NULL;
+
 	if (itemview->itemListView)
 		item_list_view_update (itemview->itemListView);
 
 	if (itemview->itemListView && itemview->node) {
-		item_list_view_enable_favicon_column (itemview->itemListView, NODE_TYPE (itemview->node)->capabilities & NODE_CAPABILITY_SHOW_ITEM_FAVICONS);
+		item_list_view_enable_favicon_column (itemview->itemListView, NODE_PROVIDER (itemview->node)->capabilities & NODE_CAPABILITY_SHOW_ITEM_FAVICONS);
 		item_list_view_set_sort_column (itemview->itemListView, itemview->node->sortColumn, itemview->node->sortReversed);
 	}
 
 	if (itemview->needsHTMLViewUpdate) {
 		itemview->needsHTMLViewUpdate = FALSE;
-		liferea_browser_update (itemview->htmlview, itemview->mode);
+
+		switch (itemview->mode) {
+			case ITEMVIEW_SINGLE_ITEM:
+				item = itemlist_get_selected ();
+				if(!item)
+					return;
+
+				node = node_from_id (item->nodeId);
+
+				direction = item_get_text_direction (item);
+				name = "item";
+				json = item_to_json (item);
+				break;
+			case ITEMVIEW_NODE_INFO:
+				node = feedlist_get_selected ();
+				if (!node)
+					return;
+
+				direction = common_get_app_direction ();
+				name = "node";
+				json = node_to_json (node);
+				break;
+		}
+
+		if (node_get_base_url (node))
+			baseURL = g_markup_escape_text ((gchar *)node_get_base_url (node), -1);
+
+		liferea_browser_set_view (itemview->htmlview, name, json, baseURL, direction);
 	}
 }
 
@@ -369,8 +388,8 @@ void
 itemview_set_layout (nodeViewType newMode)
 {
 	GtkWidget 	*previous_parent = NULL;
-	const gchar	*htmlWidgetName, *ilWidgetName, *encViewVBoxName;
-	nodePtr		node;
+	const gchar	*htmlWidgetName, *ilWidgetName;
+	Node		*node;
 	itemPtr		item;
 	nodeViewType	effectiveMode = newMode;
 
@@ -399,7 +418,6 @@ itemview_set_layout (nodeViewType newMode)
 
 	/* Prepare widgets for layout */
 	g_assert (itemview->htmlview);
-	liferea_browser_clear (itemview->htmlview);
 
 	debug (DEBUG_GUI, "Setting item list layout mode: %d (auto=%d)", effectiveMode, itemview->autoLayout);
 
@@ -407,12 +425,10 @@ itemview_set_layout (nodeViewType newMode)
 		case NODE_VIEW_MODE_NORMAL:
 			htmlWidgetName = "normalViewHtml";
 			ilWidgetName = "normalViewItems";
-			encViewVBoxName = "normalViewVBox";
 			break;
 		case NODE_VIEW_MODE_WIDE:
 			htmlWidgetName = "wideViewHtml";
 			ilWidgetName = "wideViewItems";
-			encViewVBoxName = "wideViewVBox";
 			break;
 		default:
 			g_warning("fatal: illegal viewing mode!");
@@ -441,21 +457,6 @@ itemview_set_layout (nodeViewType newMode)
 		gtk_container_add (GTK_CONTAINER (liferea_shell_lookup (ilWidgetName)), item_list_view_get_widget (itemview->itemListView));
 	}
 
-	/* Destroy previous enclosure list. */
-	if (itemview->enclosureView) {
-		gtk_widget_destroy (enclosure_list_view_get_widget (itemview->enclosureView));
-		itemview->enclosureView = NULL;
-	}
-
-	/* Create a new enclosure list GtkTreeView. */
-	if (encViewVBoxName) {
-		itemview->enclosureView = enclosure_list_view_new ();
-		gtk_grid_attach_next_to (GTK_GRID (liferea_shell_lookup (encViewVBoxName)),
-		                  enclosure_list_view_get_widget (itemview->enclosureView),
-				  NULL, GTK_POS_BOTTOM, 1,1);
-		gtk_widget_show_all (liferea_shell_lookup (encViewVBoxName));
-	}
-
 	/* Load previously selected node and/or item into new widgets */
 	if (node) {
 		itemlist_load (node);
